﻿using System;
using System.Collections.Generic;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence.Core {

  public sealed class ReverseTypeCache {

    private readonly Dictionary<int, object> typeId2SerializerMapping;
    private readonly Dictionary<Type, object> serializerInstances;
    private readonly Dictionary<int, Type> typeIds;

    public ReverseTypeCache() {
      typeId2SerializerMapping = new Dictionary<int, object>();
      serializerInstances = new Dictionary<Type, object>();
      typeIds = new Dictionary<int, Type>();
    }

    public void AddSerializer(int typeId, Type serializerType) {
      if (typeId2SerializerMapping.ContainsKey(typeId))
        return;
      object serializerInstance;
      if (!serializerInstances.TryGetValue(serializerType, out serializerInstance)) {
        if (serializerType == typeof(CachedTypeSerializer)) {
          serializerInstance = new CachedTypeSerializer(typeIds);
        } else {
          serializerInstance = serializerType.GetConstructor(System.Type.EmptyTypes).Invoke(new object[0]);
        }
        serializerInstances.Add(serializerType, serializerInstance);
      }
      typeId2SerializerMapping.Add(typeId, serializerInstance);
    }

    public void AddType(int typeId, Type type) {
      typeIds.Add(typeId, type);
    }

    public Type GetType(int id) {
      return typeIds[id];
    }

    public ICompositeSerializer GetCompositeSerializer(int typeId) {
      return (ICompositeSerializer)typeId2SerializerMapping[typeId];
    }

    public IPrimitiveSerializer GetPrimitiveSerializer(int typeId) {
      return (IPrimitiveSerializer)typeId2SerializerMapping[typeId];
    }
  }
}
